package ru.dbotthepony.mc.otm.capability;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraftforge.energy.IEnergyStorage;
import ru.dbotthepony.mc.otm.core.Fraction;

import javax.annotation.ParametersAreNonnullByDefault;

// IEnergyStorage for direct compat with Forge Energy
@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
@SuppressWarnings("unused")
public interface IMatteryEnergyStorage extends IEnergyStorage {
	// such as cables. This is something that would work only with energy storage
	Fraction extractEnergyOuter(Fraction howMuch, boolean simulate);

	// for internal needs, e.g. for work
	// CAN also be used by something that does evil
	// e.g. sucking out energy anomaly should use this
	Fraction extractEnergyInner(Fraction howMuch, boolean simulate);

	// energy is received from outside, e.g. cables
	Fraction receiveEnergyOuter(Fraction howMuch, boolean simulate);

	// energy is received from inside, e.g. generator generates power
	Fraction receiveEnergyInner(Fraction howMuch, boolean simulate);

	default Fraction extractEnergyOuter(long howMuch, boolean simulate) {
		return extractEnergyOuter(new Fraction(howMuch), simulate);
	}

	default Fraction extractEnergyOuter(int howMuch, boolean simulate) {
		return extractEnergyOuter(new Fraction(howMuch), simulate);
	}

	default Fraction receiveEnergyOuter(long howMuch, boolean simulate) {
		return receiveEnergyOuter(new Fraction(howMuch), simulate);
	}

	default Fraction receiveEnergyOuter(int howMuch, boolean simulate) {
		return receiveEnergyOuter(new Fraction(howMuch), simulate);
	}

	default Fraction extractEnergyInner(long howMuch, boolean simulate) {
		return extractEnergyInner(new Fraction(howMuch), simulate);
	}

	default Fraction extractEnergyInner(int howMuch, boolean simulate) {
		return extractEnergyInner(new Fraction(howMuch), simulate);
	}

	default Fraction receiveEnergyInner(long howMuch, boolean simulate) {
		return receiveEnergyInner(new Fraction(howMuch), simulate);
	}

	default Fraction receiveEnergyInner(int howMuch, boolean simulate) {
		return receiveEnergyInner(new Fraction(howMuch), simulate);
	}

	Fraction getBatteryLevel();

	Fraction getMaxBatteryLevel();

	default Fraction getMissingPower() {
		return getMaxBatteryLevel().minus(getBatteryLevel());
	}

	@Override
	default int receiveEnergy(int maxReceive, boolean simulate) {
		int received = receiveEnergyOuter(maxReceive, true).toInt();

		if (received == 0) {
			// Receiving only a fraction
			return 0;
		}

		return receiveEnergyOuter(new Fraction(received), simulate).toInt();
	}

	@Override
	default int extractEnergy(int maxReceive, boolean simulate) {
		int extracted = extractEnergyOuter(maxReceive, true).toInt();

		if (extracted == 0) {
			// Extracting only a fraction
			return 0;
		}

		return extractEnergyOuter(new Fraction(extracted), simulate).toInt();
	}

	@Override
	default int getEnergyStored() {
		Fraction level = getBatteryLevel();

		if (level.compareTo(MatteryCapability.INT_MAX_VALUE) < 0) {
			return level.toInt();
		}

		return Integer.MAX_VALUE;
	}

	@Override
	default int getMaxEnergyStored() {
		Fraction level = getMaxBatteryLevel();

		if (level.compareTo(MatteryCapability.INT_MAX_VALUE) < 0) {
			return level.toInt();
		}

		return Integer.MAX_VALUE;
	}

	@Override
	default boolean canExtract() {
		return extractEnergyOuter(Fraction.ONE, true).compareTo(Fraction.ZERO) > 0;
	}

	@Override
	default boolean canReceive() {
		return receiveEnergyOuter(Fraction.ONE, true).compareTo(Fraction.ZERO) > 0;
	}
}
