package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.items.CapabilityItemHandler
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.worker.BlockEntityMatteryWorker
import ru.dbotthepony.mc.otm.block.entity.worker.MachineJob
import ru.dbotthepony.mc.otm.block.entity.worker.MachineJobStatus
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage
import ru.dbotthepony.mc.otm.capability.matter.*
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.matter.MatterRegistry
import ru.dbotthepony.mc.otm.menu.MatterScannerMenu
import java.util.*

class BlockEntityMatterScanner(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntityMatteryWorker(Registry.BlockEntities.MATTER_SCANNER, p_155229_, p_155230_), IMatterGraphNode {

	val input_slot = MatteryContainer(this::setChanged, 1)

	private val inputHandler = input_slot.handler(
		{ slot: Int, stack: ItemStack? -> MatterRegistry.canDecompose(stack) },
		{ slot: Int, amount: Int, stack: ItemStack? -> is_idling }
	)

	// IMatterGraphNode
	override fun getAsMatterNode(): Graph6Node<IMatterGraphNode> {
		return matterNode
	}

	override fun onPatternAdded(state: PatternState) {
		is_idling = false
	}

	override fun onPatternRemoved(state: PatternState) {
		is_idling = false
	}

	override fun onPatternUpdated(new_state: PatternState, old_state: PatternState) {
		is_idling = false
	}
	// /IMatterGraphNode

	private var valid = true
	private val matterNode: Graph6Node<IMatterGraphNode> = Graph6Node(this)
	private var resolverNode = LazyOptional.of { this }

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) return inputHandler.get().cast()
			//if (cap === MatteryCapability.MATTER_CELL) return resolver_grid.cast()
			if (cap == MatteryCapability.MATTER_CELL) return resolverNode.cast()
		}

		return super.getCapability(cap, side)
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
		inputHandler.invalidate()
		resolverNode.invalidate()
		matterNode.destroy(::MatterNetworkGraph)
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
		inputHandler.revive()
		resolverNode = LazyOptional.of { this }
	}

	override fun setRemoved() {
		super.setRemoved()
		matterNode.destroy(::MatterNetworkGraph)
	}

	override fun getDefaultDisplayName(): Component {
		return NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu? {
		return MatterScannerMenu(containerID, inventory, this)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt.put("work_slots", input_slot.serializeNBT())
	}

	override fun load(nbt: CompoundTag) {
		input_slot.deserializeNBT(nbt["work_slots"])
		super.load(nbt)
	}

	override fun getBaseConsumption(): Fraction {
		return BASE_CONSUMPTION
	}

	override fun onJobFinish(job: MachineJob): MachineJobStatus {
		val grid = matterNode.graph as MatterNetworkGraph? ?: return MachineJobStatus(false, 100)

		val stack = job.stack()
		if (stack.isEmpty || !MatterRegistry.hasMatterValue(stack)) return MachineJobStatus()

		val get_state = grid.findPatterns(stack.item)
		var find_state: PatternState? = null

		for (state in get_state) {
			if (state.item() === stack.item) {
				if (find_state == null && state.research_percent() < 1.0) {
					find_state = state
				} else if (find_state != null && find_state.research_percent() < state.research_percent()) {
					find_state = state
				}
			}
		}

		val new_state =
			if (find_state != null) {
				PatternState(find_state.id(), stack.item, find_state.research_percent() + 0.2)
			} else {
				PatternState(UUID.randomUUID(), stack.item, 0.2)
			}

		if (grid.insertPattern(new_state, false, false).status != PatternInsertStatus.Status.FAIL) {
			return MachineJobStatus()
		} else {
			return MachineJobStatus(false, 200)
		}
	}

	override fun getNextJob(): MachineJob? {
		val grid = matterNode.graph as MatterNetworkGraph? ?: return null

		val stack = input_slot.getItem(0)
		if (stack.isEmpty || !MatterRegistry.canDecompose(stack)) return null

		val getState = grid.findPatterns(stack.item)
		var findState: PatternState? = null

		for (state in getState) {
			if (state.item() === stack.item && state.research_percent() < 1.0) {
				findState = state
			} else if (state.item() === stack.item && state.research_percent() >= 1.0) {
				return null
			}
		}

		val new_state: PatternState =
			if (findState != null) {
				PatternState(findState.id, stack.item, findState.research_percent + 0.2)
			} else {
				PatternState(UUID.randomUUID(), stack.item, 0.2)
			}

		if (grid.insertPattern(new_state, false, true).status != PatternInsertStatus.Status.FAIL) {
			val copy = stack.copy()
			copy.count = 1
			stack.shrink(1)
			input_slot.setChanged()
			return MachineJob(copy, MatterRegistry.getMatterValue(copy).toDouble() * 35000.0)
		}

		return null
	}

	init {
		energy = MatteryMachineEnergyStorage(
			this,
			MatteryMachineEnergyStorage.MachineType.WORKER,
			Fraction(40000),
			Fraction(400),
			Fraction(400)
		)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel) {
			MatterNetworkGraph.discoverFull(this, matterNode)
		}
	}

	companion object {
		private val NAME = TranslatableComponent("block.overdrive_that_matters.matter_scanner")
		private val BASE_CONSUMPTION = Fraction("40")
	}
}